@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,8 +19,13 @@
 
 This exposes the choices made by the user. Defaults will be applied here, and
 some handling of defaults.
+
 """
 
+# These are for use in option values.
+# spell-checker: ignore uiaccess,noannotations,reexecution,etherium
+# spell-checker: ignore nodocstrings,noasserts,nowarnings,norandomization
+
 import os
 import shlex
 import sys
@@ -30,6 +35,7 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.OptionParsing import isPyenvPython, parseOptions
 from nuitka.PythonFlavors import (
     isAnacondaPython,
+    isApplePython,
     isDebianPackagePython,
     isMSYS2MingwPython,
     isNuitkaPython,
@@ -37,6 +43,8 @@ from nuitka.PythonFlavors import (
 )
 from nuitka.PythonVersions import (
     getSupportedPythonVersions,
+    isDebugPython,
+    python_version,
     python_version_str,
 )
 from nuitka.utils.Execution import getExecutablePath
@@ -52,6 +60,7 @@ from nuitka.utils.Utils import (
     getOS,
     hasOnefileSupportedOS,
     hasStandaloneSupportedOS,
+    isDebianBasedLinux,
     isFreeBSD,
     isLinux,
     isMacOS,
@@ -64,9 +73,61 @@ positional_args = None
 extra_args = []
 is_nuitka_run = None
 is_debug = None
-is_nondebug = None
-is_fullcompat = None
+is_non_debug = None
+is_full_compat = None
 is_report_missing = None
+is_verbose = None
+
+
+def _checkSpec(value, arg_name):
+    if "%COMPANY%" in value and not getWindowsCompanyName():
+        Tracing.options_logger.sysexit(
+            "Using value '%%COMPANY%%' in '%s=%s' value without being specified."
+            % (arg_name, value)
+        )
+
+
+def _checkOnefileTargetSpec():
+    options.is_onefile_tempdir = True
+
+    _checkSpec(options.onefile_tempdir_spec, arg_name="--onefile-tempdir-spec")
+
+    if os.path.normpath(options.onefile_tempdir_spec) == ".":
+        Tracing.options_logger.sysexit(
+            """\
+Error, using '.' as a value for '--onefile-tempdir-spec' is not supported,
+you cannot unpack the onefile payload into the same directory as the binary,
+as that would overwrite it and cause locking issues as well."""
+        )
+
+    if options.onefile_tempdir_spec.count("%") % 2 != 0:
+        Tracing.options_logger.warning(
+            """Unmatched '%%' is suspicious for '--onefile-tempdir-spec' and may \
+not do what you want it to do: '%s'"""
+            % options.onefile_tempdir_spec
+        )
+
+    if options.onefile_tempdir_spec.count("%") == 0:
+        Tracing.options_logger.warning(
+            """Not using any variables for '--onefile-tempdir-spec' should only be \
+done if your program absolutely needs to be in the same path always: '%s'"""
+            % options.onefile_tempdir_spec
+        )
+
+    if os.path.isabs(options.onefile_tempdir_spec):
+        Tracing.options_logger.warning(
+            """Using an absolute path should be avoided unless you are targeting a \
+very well known environment: anchor with e.g. %%TEMP%%, %%CACHE_DIR%% is recommended: '%s'"""
+            % options.onefile_tempdir_spec
+        )
+    elif not options.onefile_tempdir_spec.startswith(
+        ("%TEMP%", "%HOME%", "%CACHE_DIR%")
+    ):
+        Tracing.options_logger.warning(
+            """Using an relative to the executable should be avoided unless you are targeting a \
+very well known environment, anchor with e.g. %%TEMP%%, %%CACHE_DIR%% is recommended: '%s'"""
+            % options.onefile_tempdir_spec
+        )
 
 
 def parseArgs():
@@ -76,7 +137,8 @@ def parseArgs():
     """
     # singleton with many cases checking the options right away.
     # pylint: disable=global-statement,too-many-branches,too-many-locals,too-many-statements
-    global is_nuitka_run, options, positional_args, extra_args, is_debug, is_nondebug, is_fullcompat, is_report_missing
+    global is_nuitka_run, options, positional_args, extra_args, is_debug, is_non_debug
+    global is_full_compat, is_report_missing, is_verbose
 
     if os.name == "nt":
         # Windows store Python's don't allow looking at the python, catch that.
@@ -93,8 +155,11 @@ def parseArgs():
     )
 
     is_debug = _isDebug()
-    is_nondebug = not is_debug
-    is_fullcompat = _isFullCompat()
+    is_non_debug = not is_debug
+    is_full_compat = _isFullCompat()
+
+    if hasattr(options, "experimental"):
+        _experimental.update(options.experimental)
 
     # TODO: Have dedicated option for it.
     is_report_missing = is_debug
@@ -123,6 +188,8 @@ def parseArgs():
 
         options.verbose = True
 
+    is_verbose = options.verbose
+
     Tracing.optimization_logger.is_quiet = not options.verbose
 
     if options.show_inclusion_output:
@@ -139,15 +206,20 @@ def parseArgs():
     if options.is_onefile:
         options.is_standalone = True
 
-    # Provide a tempdir spec implies onefile tempdir, even on Linux.
-    if options.onefile_tempdir_spec:
-        options.is_onefile_tempdir = True
+    # Standalone implies no_site build
+    if options.is_standalone:
+        options.python_flags.insert(0, "no_site")
 
+    # Check onefile tempdir spec.
+    if options.onefile_tempdir_spec:
+        _checkOnefileTargetSpec()
+
+    # Provide a tempdir spec implies onefile tempdir, even on Linux.
     # Standalone mode implies an executable, not importing "site" module, which is
     # only for this machine, recursing to all modules, and even including the
     # standard library.
     if options.is_standalone:
-        if not options.executable:
+        if options.module_mode:
             Tracing.options_logger.sysexit(
                 """\
 Error, conflicting options, cannot make standalone module, only executable.
@@ -211,7 +283,15 @@ standalone where there is a sane default used inside the dist folder."""
 
     if isLinux():
         if len(getIconPaths()) > 1:
-            Tracing.options_logger.sysexit("Error, can only use one icon on Linux.")
+            Tracing.options_logger.sysexit(
+                "Error, can only use one icon file on Linux."
+            )
+
+    if isMacOS():
+        if len(getIconPaths()) > 1:
+            Tracing.options_logger.sysexit(
+                "Error, can only use one icon file on macOS."
+            )
 
     for icon_path in getIconPaths():
         if "#" in icon_path and isWin32Windows():
@@ -286,11 +366,6 @@ standalone where there is a sane default used inside the dist folder."""
             "Conflicting options '--follow-imports' and '--nofollow-imports' given."
         )
 
-    if getShallIncludePackageData() and not isStandaloneMode():
-        Tracing.options_logger.sysexit(
-            "Error, package data files are only included in standalone or onefile mode."
-        )
-
     for module_pattern in getShallIncludePackageData():
         if (
             module_pattern.startswith("-")
@@ -323,14 +398,14 @@ standalone where there is a sane default used inside the dist folder."""
                 % module_pattern
             )
 
-    for data_file in options.data_files:
-        if "=" not in data_file:
+    for data_file_desc in options.data_files:
+        if "=" not in data_file_desc:
             Tracing.options_logger.sysexit(
                 "Error, malformed data file description, must specify relative target path separated with '='."
             )
 
-        if data_file.count("=") == 1:
-            src, dst = data_file.split("=", 1)
+        if data_file_desc.count("=") == 1:
+            src, dst = data_file_desc.split("=", 1)
 
             filenames = resolveShellPatternToFilenames(src)
 
@@ -340,7 +415,7 @@ standalone where there is a sane default used inside the dist folder."""
                     % src
                 )
         else:
-            src, dst, pattern = data_file.split("=", 2)
+            src, dst, pattern = data_file_desc.split("=", 2)
 
             filenames = resolveShellPatternToFilenames(os.path.join(src, pattern))
 
@@ -352,7 +427,7 @@ standalone where there is a sane default used inside the dist folder."""
         if os.path.isabs(dst):
             Tracing.options_logger.sysexit(
                 "Error, must specify relative target path for data file, not absolute path '%s'."
-                % data_file
+                % data_file_desc
             )
 
     for data_dir in options.data_dirs:
@@ -375,11 +450,6 @@ standalone where there is a sane default used inside the dist folder."""
                 % (dst, data_dir)
             )
 
-    if (options.data_files or options.data_dirs) and not isStandaloneMode():
-        Tracing.options_logger.sysexit(
-            "Error, data files are only included in standalone or onefile mode."
-        )
-
     for pattern in getShallFollowExtraFilePatterns():
         if os.path.isdir(pattern):
             Tracing.options_logger.sysexit(
@@ -399,10 +469,16 @@ might be missing required packages. Disable with --static-libpython=no" if you d
 want to install it."""
         )
 
-    if isStandaloneMode() and isMacOS() and sys.executable.startswith("/usr/bin/"):
-        Tracing.options_logger.sysexit(
-            "Error, Apple Python from macOS is not supported, use e.g. CPython instead."
-        )
+    if isApplePython():
+        if isStandaloneMode():
+            Tracing.options_logger.sysexit(
+                "Error, for standalone mode, Apple Python from macOS is not supported, use e.g. CPython instead."
+            )
+
+        if str is bytes:
+            Tracing.options_logger.sysexit(
+                "Error, Apple Python 2.7 from macOS is not usable as per Apple decision, use e.g. CPython 2.7 instead."
+            )
 
     if isStandaloneMode() and isLinux() and getExecutablePath("patchelf") is None:
         Tracing.options_logger.sysexit(
@@ -454,9 +530,19 @@ def commentArgs():
                 % default_reference_mode
             )
 
+    default_mode_name_mode = "runtime" if shallMakeModule() else "original"
+
+    if getModuleNameMode() is None:
+        options.module_name_mode = default_mode_name_mode
+    elif getModuleNameMode() == default_mode_name_mode:
+        Tracing.options_logger.info(
+            "Using module name mode '%s' need not be specified."
+            % default_mode_name_mode
+        )
+
     # TODO: Not all of these are usable with MSYS2 really, split those off.
     if getOS() != "Windows":
-        # Too many Windows specific options clearly, pylint: disable=too-many-boolean-expressions
+        # Too many Windows specific options clearly
         if (
             getWindowsIconExecutablePath()
             or shallAskForWindowsAdminRights()
@@ -465,10 +551,7 @@ def commentArgs():
             or getWindowsProductName()
             or getWindowsProductVersion()
             or getWindowsFileVersion()
-            or getForcedStderrPath()  # not yet for other platforms
-            or getForcedStdoutPath()
             or getWindowsSplashScreen()
-            or getIntendedPythonArch()
         ):
             Tracing.options_logger.warning(
                 "Using Windows specific options has no effect on other platforms."
@@ -484,6 +567,15 @@ def commentArgs():
             "Requesting both Windows specific compilers makes no sense."
         )
 
+    if getMsvcVersion() and getMsvcVersion() not in ("list", "latest"):
+        if getMsvcVersion().count(".") != 1 or not all(
+            x.isdigit() for x in getMsvcVersion().split(".")
+        ):
+            Tracing.options_logger.sysexit(
+                "For --msvc only values 'latest', 'info', and 'X.Y' values are allowed, but not %r."
+                % getMsvcVersion()
+            )
+
     if isOnefileMode():
         standalone_mode = "onefile"
     elif isStandaloneMode():
@@ -497,6 +589,14 @@ def commentArgs():
             % getOS()
         )
 
+    if options.follow_all and shallMakeModule():
+        Tracing.optimization_logger.sysexit(
+            """\
+In module mode you must follow modules more selectively, and e.g. should \
+not include standard library or all foreign modules or else it will fail \
+to work. You can selectively add them with '--follow-import-to=name' though."""
+        )
+
     if options.follow_all and standalone_mode:
         Tracing.options_logger.info(
             "Following all imports is the default for %s mode and need not be specified."
@@ -509,6 +609,11 @@ def commentArgs():
             % standalone_mode
         )
 
+    if options.follow_stdlib and not standalone_mode:
+        Tracing.options_logger.warning(
+            "Following imports to stdlib is unlikely to work without --standalone/--onefile and should not be specified."
+        )
+
     if (
         not shallDumpBuiltTreeXML()
         and not standalone_mode
@@ -582,6 +687,39 @@ but errors may happen."""
                 "The '--execute-with-pythonpath' option has no effect without '--run' option."
             )
 
+    # Check if the fallback is used, except for Python2 on Windows, where we cannot
+    # have it.
+    if hasattr(OrderedSet, "is_fallback") and not (
+        isWin32Windows() and python_version < 0x360
+    ):
+        # spell-checker: ignore orderedset
+        Tracing.general.warning(
+            """\
+Using very slow fallback for ordered sets, please install '%s' \
+PyPI package for best Python compile time performance."""
+            % ('ordered-set' if python_version >= 0x3A0 else 'orderedset')
+        )
+
+    if shallUsePythonDebug() and not isDebugPython():
+        Tracing.general.sysexit(
+            """\
+Error, for using the debug Python version, you need to run it will that version
+and not with the non-debug version.
+"""
+        )
+
+    filename = getPositionalArgs()[0]
+    if not os.path.exists(filename):
+        Tracing.general.sysexit("Error, file '%s' is not found." % filename)
+
+    if (
+        shallMakeModule()
+        and os.path.normcase(os.path.basename(filename)) == "__init__.py"
+    ):
+        Tracing.general.sysexit(
+            "Error, to compile a package, specify its directory name, not the '__init__.py'."
+        )
+
 
 def isVerbose():
     """:returns: bool derived from ``--verbose``"""
@@ -629,9 +767,19 @@ def getFileReferenceMode():
     return options.file_reference_mode
 
 
+def getModuleNameMode():
+    """*str*, one of "runtime", "original", coming from ``--module-name-choice``
+
+    Notes:
+        Defaults to runtime for modules and packages, otherwise original is kept.
+    """
+
+    return options.module_name_mode
+
+
 def shallMakeModule():
     """:returns: bool derived from ``--module``"""
-    return not options.executable
+    return options.module_mode
 
 
 def shallCreatePyiFile():
@@ -707,18 +855,18 @@ def getShallIncludePackageData():
 
 
 def getShallIncludeDataFiles():
-    """*list*, items of ``--include-data-file=``"""
-    for data_file in options.data_files:
-        if data_file.count("=") == 1:
-            src, dest = data_file.split("=", 1)
+    """*list*, items of ``--include-data-files=``"""
+    for data_file_desc in options.data_files:
+        if data_file_desc.count("=") == 1:
+            src, dest = data_file_desc.split("=", 1)
 
             for pattern in _splitShellPattern(src):
-                yield pattern, None, dest, data_file
+                yield pattern, None, dest, data_file_desc
         else:
-            src, dest, pattern = data_file.split("=", 2)
+            src, dest, pattern = data_file_desc.split("=", 2)
 
             for pattern in _splitShellPattern(pattern):
-                yield os.path.join(src, pattern), src, dest, data_file
+                yield os.path.join(src, pattern), src, dest, data_file_desc
 
 
 def getShallIncludeDataDirs():
@@ -729,6 +877,12 @@ def getShallIncludeDataDirs():
         yield src, dest
 
 
+def getShallNotIncludeDataFilePatterns():
+    """*list*, items of ``--noinclude-data-files=``"""
+
+    return options.data_files_inhibited
+
+
 def shallWarnImplicitRaises():
     """:returns: bool derived from ``--warn-implicit-exceptions``"""
     return options.warn_implicit_exceptions
@@ -749,7 +903,7 @@ def _isDebug():
     return options is not None and (options.debug or options.debugger)
 
 
-def isPythonDebug():
+def shallUsePythonDebug():
     """:returns: bool derived from ``--python-debug`` or ``sys.flags.debug``
 
     Passed to Scons as ``python_debug`` so it can consider it when picking
@@ -761,17 +915,17 @@ def isPythonDebug():
     return options.python_debug or sys.flags.debug
 
 
-def isUnstripped():
-    """:returns: bool derived from ``--unstripped`` or ``--profile``
+def isUnstriped():
+    """:returns: bool derived from ``--unstriped`` or ``--profile``
 
     A binary is called stripped when debug information is not present, an
-    unstripped when it is present. For profiling and debugging it will be
-    necessary, but it doesn*t enable debug checks like ``--debug`` does.
+    unstriped when it is present. For profiling and debugging it will be
+    necessary, but it doesn't enable debug checks like ``--debug`` does.
 
-    Passed to Scons as ``unstripped_mode`` to it can ask the linker to
+    Passed to Scons as ``unstriped_mode`` to it can ask the linker to
     include symbol information.
     """
-    return options.unstripped or options.profile
+    return options.unstriped or options.profile
 
 
 def isProfile():
@@ -780,8 +934,8 @@ def isProfile():
 
 
 def shallCreateGraph():
-    """:returns: bool derived from ``--graph``"""
-    return options.graph
+    """:returns: bool derived from ``--internal-graph``"""
+    return options.internal_graph
 
 
 def getOutputFilename():
@@ -842,9 +996,10 @@ def _shallUseStaticLibPython():
         )
 
         if (
-            isDebianPackagePython()
+            isDebianBasedLinux()
+            and isDebianPackagePython()
             and isDebianSuitableForStaticLinking()
-            and not isPythonDebug()
+            and not shallUsePythonDebug()
         ):
             return True, "Nuitka on Debian-Python needs package '%s' installed." % (
                 "python2-dev" if str is bytes else "python3-dev"
@@ -856,7 +1011,10 @@ def _shallUseStaticLibPython():
         # For Anaconda default to trying static lib python library, which
         # normally is just not available or if it is even unusable.
         if isAnacondaPython() and not isMacOS() and not isWin32Windows():
-            return True, "Nuitka on Anaconda needs package 'libpython' installed."
+            return (
+                True,
+                "Nuitka on Anaconda needs package for static libpython installed. Execute 'conda install libpython-static'.",
+            )
 
         if isPyenvPython():
             return True, "Nuitka on pyenv should not use '--enable-shared'."
@@ -910,6 +1068,15 @@ def shallTreatUninstalledPython():
     return isUninstalledPython()
 
 
+def shallCreateCmdFileForExecution():
+    """*bool* = derived from Python installation and modes
+
+    Notes: Mostly for accelerated mode on Windows with uninstalled python, to
+    make sure they find their Python DLL.
+    """
+    return isWin32Windows() and shallTreatUninstalledPython()
+
+
 def isShowScons():
     """:returns: bool derived from ``--show-scons``"""
     return options.show_scons
@@ -959,20 +1126,32 @@ def getMsvcVersion():
 
 
 def shallDisableCCacheUsage():
-    """:returns: bool derived from ``disable-ccache``"""
+    """:returns: bool derived from ``--disable-ccache``"""
     return options.disable_ccache
 
 
+def shallDisableBytecodeCacheUsage():
+    """:returns: bool derived from ``--disable-bytecode-cache``"""
+    return options.disable_bytecode_cache
+
+
 def shallDisableConsoleWindow():
-    """:returns: bool derived from ``--win-disable-console or ``--macos-disable-console``"""
+    """:returns: None (not given), False, or True derived from ``disable-console or ``--enable-console``"""
     return options.disable_console
 
 
+def mayDisableConsoleWindow():
+    """:returns: bool derived from platform support of disabling the console,"""
+
+    # TODO: What about MSYS2?
+    return isWin32Windows() or isMacOS()
+
+
 def _isFullCompat():
     """:returns: bool derived from ``--full-compat``
 
     Notes:
-        Code should should use "Options.is_fullcompat" instead, this
+        Code should should use "Options.is_full_compat" instead, this
         is only used to initialize that value.
     """
     return options is not None and not options.improved
@@ -998,16 +1177,7 @@ def isRemoveBuildDir():
     return options.remove_build and not options.generate_c_only
 
 
-def getIntendedPythonArch():
-    """:returns: str, one of ``"x86"``, ``"x86_64"`` or ``None``
-
-    Notes: This is only available on Windows, on other platforms
-    it will be `None`
-    """
-    return options.python_arch if isWin32Windows() else None
-
-
-experimental = set()
+_experimental = set()
 
 
 def isExperimental(indication):
@@ -1018,19 +1188,15 @@ def isExperimental(indication):
     Returns:
         bool
     """
-    return (
-        indication in experimental
-        or hasattr(options, "experimental")
-        and indication in options.experimental
-    )
+    return indication in _experimental
 
 
 def enableExperimental(indication):
-    experimental.add(indication)
+    _experimental.add(indication)
 
 
 def disableExperimental(indication):
-    experimental.remove(indication)
+    _experimental.remove(indication)
 
 
 def getExperimentalIndications():
@@ -1056,6 +1222,11 @@ def isOnefileMode():
     return options.is_onefile
 
 
+def isAcceleratedMode():
+    """:returns: bool derived from ``--standalone`` and `--module`"""
+    return not isStandaloneMode() and not shallMakeModule()
+
+
 def isOnefileTempDirMode():
     """:returns: bool derived from ``--onefile-tempdir`` and OS
 
@@ -1108,14 +1279,10 @@ def getPythonPgoUnseenModulePolicy():
     return options.python_pgo_policy_unused_module
 
 
-def getOnefileTempDirSpec(use_default):
-    if use_default:
-        return (
-            options.onefile_tempdir_spec
-            or "%TEMP%" + os.path.sep + "onefile_%PID%_%TIME%"
-        )
-    else:
-        return options.onefile_tempdir_spec
+def getOnefileTempDirSpec():
+    return (
+        options.onefile_tempdir_spec or "%TEMP%" + os.path.sep + "onefile_%PID%_%TIME%"
+    )
 
 
 def getIconPaths():
@@ -1125,6 +1292,7 @@ def getIconPaths():
 
     # Check if Linux icon requirement is met.
     if isLinux() and not result and isOnefileMode():
+        # spell-checker: ignore pixmaps
         default_icons = (
             "/usr/share/pixmaps/python%s.xpm" % python_version_str,
             "/usr/share/pixmaps/python%s.xpm" % sys.version_info[0],
@@ -1190,7 +1358,7 @@ def _parseWindowsVersionNumber(value):
 
         r = tuple(int(d) for d in parts)
         assert min(r) >= 0
-        assert max(r) < 2 ** 16
+        assert max(r) < 2**16
         return r
     else:
         return None
@@ -1221,9 +1389,24 @@ def getWindowsProductName():
     return options.windows_product_name
 
 
+def getMacOSTargetArch():
+    """:returns: str enum ("universal", "arm64", "x86_64") derived from ``--macos-target-arch`` value"""
+    macos_target_arch = options.macos_target_arch or "native"
+
+    if macos_target_arch == "native":
+        macos_target_arch = getArchitecture()
+
+    return macos_target_arch
+
+
 def shallCreateAppBundle():
-    """*bool* shall create an application bundle"""
-    return options.macos_create_bundle
+    """*bool* shall create an application bundle, derived from ``--macos-create-app-bundle`` value"""
+    return options.macos_create_bundle and isMacOS()
+
+
+def getMacOSSigningIdentity():
+    """*str* value to use as identity for codesign, derived from ``--macos-sign-identity`` value"""
+    return options.macos_sign_identity
 
 
 def getMacOSAppName():
@@ -1241,6 +1424,16 @@ def getMacOSAppVersion():
     return options.macos_app_version
 
 
+def getMacOSAppProtectedResourcesAccesses():
+    """*list* key, value for protected resources of the app to use for bundle"""
+    for macos_protected_resource in options.macos_protected_resources:
+        yield macos_protected_resource.split(":", 1)
+
+
+def getAppImageCompression():
+    return options.app_image_compression
+
+
 _python_flags = None
 
 
@@ -1258,6 +1451,9 @@ def _getPythonFlags():
             for part in parts.split(","):
                 if part in ("-S", "nosite", "no_site"):
                     _python_flags.add("no_site")
+                elif part in ("site"):
+                    if "no_site" in _python_flags:
+                        _python_flags.remove("no_site")
                 elif part in (
                     "-R",
                     "static_hashes",
@@ -1278,6 +1474,8 @@ def _getPythonFlags():
                     _python_flags.add("no_asserts")
                 elif part in ("no_annotations", "noannotations"):
                     _python_flags.add("no_annotations")
+                elif part in ("unbuffered", "-u"):
+                    _python_flags.add("unbuffered")
                 elif part in ("-m", "package_mode"):
                     _python_flags.add("package_mode")
                 else:
@@ -1328,6 +1526,12 @@ def hasPythonFlagNoRandomization():
     return "no_randomization" in _getPythonFlags()
 
 
+def hasPythonFlagUnbuffered():
+    """*bool* = "package_mode", "-m" in python flags given"""
+
+    return "unbuffered" in _getPythonFlags()
+
+
 def hasPythonFlagPackageMode():
     """*bool* = "package_mode", "-m" in python flags given"""
 
@@ -1359,9 +1563,7 @@ def shallNotStoreDependsExeCachedResults():
 def getPluginNameConsideringRenames(plugin_name):
     """Name of the plugin with renames considered."""
 
-    if plugin_name == "qt-plugins":
-        return "pyqt5"
-    elif plugin_name == "etherium":
+    if plugin_name == "etherium":
         return "ethereum"
     else:
         return plugin_name
@@ -1437,7 +1639,7 @@ def shallCompileWithoutBuildDirectory():
     return not shallRunInDebugger()
 
 
-def shallPreferSourcecodeOverExtensionModules():
+def shallPreferSourceCodeOverExtensionModules():
     """*bool* prefer source code over extension modules if both are there"""
     return options is not None and options.prefer_source_code
 
@@ -1457,9 +1659,9 @@ def getForcedStderrPath():
     return options.force_stderr_spec
 
 
-def shallPersistModifications():
-    """*bool* write plugin source changes to disk"""
-    return options is not None and options.persist_source_changes
+def shallShowSourceModifications():
+    """*bool* display plugin source changes"""
+    return options is not None and options.show_source_changes
 
 
 def isLowMemory():
@@ -1470,3 +1672,8 @@ def isLowMemory():
 def getCompilationReportFilename():
     """*str* filename to write XML report of compilation to"""
     return options.compilation_report_filename
+
+
+def getUserProvidedYamlFiles():
+    """*list* files with user provided Yaml files"""
+    return options.user_yaml_files
