@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,6 +30,7 @@ from nuitka.nodes.AttributeNodesGenerated import (
     attribute_classes,
     attribute_typed_classes,
 )
+from nuitka.nodes.BytesNodes import getBytesOperationClasses
 from nuitka.nodes.StrNodes import getStrOperationClasses
 from nuitka.plugins.Plugins import Plugins
 from nuitka.utils.CStrings import encodePythonStringToC
@@ -190,6 +191,7 @@ from .ImportCodes import (
     generateImportNameCode,
     generateImportStarCode,
 )
+from .InjectCCodes import generateInjectCCode
 from .IntegerCodes import (
     generateBuiltinInt1Code,
     generateBuiltinInt2Code,
@@ -207,6 +209,7 @@ from .IteratorCodes import (
     generateBuiltinNext2Code,
     generateSpecialUnpackCode,
     generateUnpackCheckCode,
+    generateUnpackCheckFromIteratedCode,
 )
 from .ListCodes import (
     generateBuiltinListCode,
@@ -229,6 +232,7 @@ from .LoopCodes import (
     generateLoopCode,
     generateLoopContinueCode,
 )
+from .MatchCodes import generateMatchArgsCode
 from .ModuleCodes import (
     generateModuleAttributeCode,
     generateModuleAttributeFileCode,
@@ -240,6 +244,19 @@ from .OperationCodes import (
     generateOperationNotCode,
     generateOperationUnaryCode,
 )
+from .PackageResourceCodes import (
+    generateImportlibMetadataBackportVersionCallCode,
+    generateImportlibMetadataVersionCallCode,
+    generateImportlibResourcesReadBinaryCallCode,
+    generateImportlibResourcesReadTextCallCode,
+    generateOsUnameCallCode,
+    generatePkglibGetDataCallCode,
+    generatePkgResourcesDistributionValueCode,
+    generatePkgResourcesGetDistributionCallCode,
+    generatePkgResourcesRequireCallCode,
+    generatePkgResourcesResourceStreamCallCode,
+    generatePkgResourcesResourceStringCallCode,
+)
 from .PrintCodes import generatePrintNewlineCode, generatePrintValueCode
 from .RaisingCodes import (
     generateRaiseCode,
@@ -278,7 +295,9 @@ from .StringCodes import (
     generateBuiltinOrdCode,
     generateBuiltinStrCode,
     generateBuiltinUnicodeCode,
-    generateStringContenationCode,
+    generateBytesOperationCode,
+    generateStrFormatMethodCode,
+    generateStringConcatenationCode,
     generateStrOperationCode,
 )
 from .SubscriptCodes import (
@@ -692,7 +711,8 @@ addExpressionDispatchDict(
         "EXPRESSION_FUNCTION_ERROR_STR": generateFunctionErrorStrCode,
         "EXPRESSION_IMPORT_MODULE_FIXED": generateImportModuleFixedCode,
         "EXPRESSION_IMPORT_MODULE_HARD": generateImportModuleHardCode,
-        "EXPRESSION_IMPORT_MODULE_NAME_HARD": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORT_MODULE_NAME_HARD_MAYBE_EXISTS": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORT_MODULE_NAME_HARD_EXISTS": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_IMPORT_MODULE_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_IMPORT_MODULE_CALL": generateImportlibImportCallCode,
         "EXPRESSION_IMPORT_NAME": generateImportNameCode,
@@ -752,6 +772,7 @@ addExpressionDispatchDict(
         # TODO: Rename to make more clear it is an outline
         "EXPRESSION_CLASS_BODY": generateFunctionOutlineCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP": generateSubscriptLookupCode,
+        "EXPRESSION_SUBSCRIPT_LOOKUP_FOR_UNPACK": generateSubscriptLookupCode,
         "EXPRESSION_SUBSCRIPT_CHECK": generateSubscriptCheckCode,
         "EXPRESSION_SLICE_LOOKUP": generateSliceLookupCode,
         "EXPRESSION_SET_OPERATION_UPDATE": generateSetOperationUpdateCode,
@@ -769,7 +790,7 @@ addExpressionDispatchDict(
         "EXPRESSION_ASYNC_ITER": generateAsyncIterCode,
         "EXPRESSION_ASYNC_NEXT": generateAsyncNextCode,
         "EXPRESSION_SELECT_METACLASS": generateSelectMetaclassCode,
-        "EXPRESSION_STRING_CONCATENATION": generateStringContenationCode,
+        "EXPRESSION_STRING_CONCATENATION": generateStringConcatenationCode,
         "EXPRESSION_BUILTIN_FORMAT": generateBuiltinFormatCode,
         "EXPRESSION_BUILTIN_ASCII": generateBuiltinAsciiCode,
         "EXPRESSION_LOCALS_VARIABLE_CHECK": generateLocalsDictVariableCheckCode,
@@ -777,6 +798,29 @@ addExpressionDispatchDict(
         "EXPRESSION_LOCALS_VARIABLE_REF": generateLocalsDictVariableRefCode,
         "EXPRESSION_RAISE_EXCEPTION": generateRaiseExpressionCode,
         "EXPRESSION_NUITKA_LOADER_CREATION": generateNuitkaLoaderCreationCode,
+        "EXPRESSION_PKGLIB_GET_DATA_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_REQUIRE_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_GET_DISTRIBUTION_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STRING_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STREAM_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_DISTRIBUTION_VALUE_REF": generatePkgResourcesDistributionValueCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_BINARY_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_TEXT_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_METADATA_VERSION_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_VERSION_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_OS_UNAME_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKGLIB_GET_DATA_CALL": generatePkglibGetDataCallCode,
+        "EXPRESSION_PKG_RESOURCES_REQUIRE_CALL": generatePkgResourcesRequireCallCode,
+        "EXPRESSION_PKG_RESOURCES_GET_DISTRIBUTION_CALL": generatePkgResourcesGetDistributionCallCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STRING_CALL": generatePkgResourcesResourceStringCallCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STREAM_CALL": generatePkgResourcesResourceStreamCallCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_BINARY_CALL": generateImportlibResourcesReadBinaryCallCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_TEXT_CALL": generateImportlibResourcesReadTextCallCode,
+        "EXPRESSION_IMPORTLIB_METADATA_VERSION_CALL": generateImportlibMetadataVersionCallCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_VERSION_CALL": generateImportlibMetadataBackportVersionCallCode,
+        "EXPRESSION_OS_UNAME_CALL": generateOsUnameCallCode,
+        "EXPRESSION_MATCH_ARGS": generateMatchArgsCode,
+        "EXPRESSION_STR_OPERATION_FORMAT": generateStrFormatMethodCode,
     }
 )
 
@@ -790,18 +834,30 @@ addExpressionDispatchDict(
     dict((cls.kind, generateAttributeLookupCode) for cls in attribute_typed_classes)
 )
 
+# Add code generation for the EXPRESSION_STR_OPERATION_* nodes.
 addExpressionDispatchDict(
     dict((cls.kind, generateStrOperationCode) for cls in getStrOperationClasses())
 )
 
+# Add code generation for the EXPRESSION_BYTES_OPERATION_* nodes.
+addExpressionDispatchDict(
+    dict((cls.kind, generateBytesOperationCode) for cls in getBytesOperationClasses())
+)
+
 
 setStatementDispatchDict(
     {
-        "STATEMENT_ASSIGNMENT_VARIABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_GENERIC": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_MUTABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_IMMUTABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_ITERATOR": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_FROM_VARIABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_FROM_TEMP_VARIABLE": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_ATTRIBUTE": generateAssignmentAttributeCode,
         "STATEMENT_ASSIGNMENT_SUBSCRIPT": generateAssignmentSubscriptCode,
         "STATEMENT_ASSIGNMENT_SLICE": generateAssignmentSliceCode,
-        "STATEMENT_DEL_VARIABLE": generateDelVariableCode,
+        "STATEMENT_DEL_VARIABLE_TOLERANT": generateDelVariableCode,
+        "STATEMENT_DEL_VARIABLE_INTOLERANT": generateDelVariableCode,
         "STATEMENT_DEL_ATTRIBUTE": generateDelAttributeCode,
         "STATEMENT_DEL_SUBSCRIPT": generateDelSubscriptCode,
         "STATEMENT_DEL_SLICE": generateDelSliceCode,
@@ -835,6 +891,7 @@ setStatementDispatchDict(
         "STATEMENT_RAISE_EXCEPTION_IMPLICIT": generateRaiseCode,
         "STATEMENT_RERAISE_EXCEPTION": generateReraiseCode,
         "STATEMENT_SPECIAL_UNPACK_CHECK": generateUnpackCheckCode,
+        "STATEMENT_SPECIAL_UNPACK_CHECK_FROM_ITERATED": generateUnpackCheckFromIteratedCode,
         "STATEMENT_EXEC": generateExecCode,
         "STATEMENT_LOCALS_DICT_SYNC": generateLocalsDictSyncCode,
         "STATEMENT_SET_LOCALS": generateSetLocalsDictCode,
@@ -843,5 +900,6 @@ setStatementDispatchDict(
         "STATEMENT_PRESERVE_FRAME_EXCEPTION": generateFramePreserveExceptionCode,
         "STATEMENT_RESTORE_FRAME_EXCEPTION": generateFrameRestoreExceptionCode,
         "STATEMENT_PUBLISH_EXCEPTION": generateExceptionPublishCode,
+        "STATEMENT_INJECT_C_CODE": generateInjectCCode,
     }
 )
