@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,12 +26,13 @@ from nuitka.utils.Jinja2 import renderTemplateFromString
 from .CodeHelpers import (
     assignConstantNoneResult,
     decideConversionCheckNeeded,
+    generateChildExpressionCode,
     generateChildExpressionsCode,
     generateExpressionCode,
     withCleanupFinally,
     withObjectCodeTemporaryAssignment,
 )
-from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCodes
+from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCode
 from .PythonAPICodes import (
     generateCAPIObjectCode,
     generateCAPIObjectCode0,
@@ -122,15 +123,12 @@ def _getDictionaryCreationCode(to_name, pairs, emit, context):
     dict_key_name = context.allocateTempName("dict_key")
     dict_value_name = context.allocateTempName("dict_value")
 
-    is_hashable_key = [pair.subnode_key.isKnownToBeHashable() for pair in pairs]
+    is_hashable_key = [pair.isKeyKnownToBeHashable() for pair in pairs]
 
     # Does this dictionary build need an exception handling at all.
     if all(is_hashable_key):
         for pair in pairs[1:]:
-            if pair.subnode_key.mayRaiseException(BaseException):
-                needs_exception_exit = True
-                break
-            if pair.subnode_value.mayRaiseException(BaseException):
+            if pair.mayRaiseException(BaseException):
                 needs_exception_exit = True
                 break
         else:
@@ -141,15 +139,18 @@ def _getDictionaryCreationCode(to_name, pairs, emit, context):
     def generateValueCode(dict_value_name, pair):
         generateExpressionCode(
             to_name=dict_value_name,
-            expression=pair.subnode_value,
+            expression=pair.getValueNode(),
             emit=emit,
             context=context,
         )
 
+    # TODO: There must be a way to avoid using a node like getKeyNode does, to
+    # create the pair creation code, maybe this kind of virtual constants can
+    # be asked, but then this code should be shared.
     def generateKeyCode(dict_key_name, pair):
         generateExpressionCode(
             to_name=dict_key_name,
-            expression=pair.subnode_key,
+            expression=pair.getKeyNode(),
             emit=emit,
             context=context,
         )
@@ -176,7 +177,7 @@ def _getDictionaryCreationCode(to_name, pairs, emit, context):
 
     key_needs_release, value_needs_release = generatePairCode(pairs[0])
 
-    # Create dictionary presized.
+    # Create dictionary pre-sized.
     emit("%s = _PyDict_NewPresized( %d );" % (to_name, pairs_count))
 
     with withCleanupFinally(
@@ -403,40 +404,16 @@ def generateDictOperationPop3Code(to_name, expression, emit, context):
     )
 
 
-def generateDictOperationUpdate2Code(to_name, expression, emit, context):
-    dict_name, iterable_name = generateChildExpressionsCode(
-        expression=expression, emit=emit, context=context
-    )
-
-    res_name = context.getIntResName()
-
-    emit("assert(PyDict_Check(%s));" % dict_name)
-    emit("%s = PyDict_Update(%s, %s);" % (res_name, dict_name, iterable_name))
-
-    getErrorExitBoolCode(
-        condition="%s != 0" % res_name,
-        release_names=(dict_name, iterable_name),
-        needs_check=expression.mayRaiseException(BaseException),
-        emit=emit,
-        context=context,
-    )
-
-    assignConstantNoneResult(to_name, emit, context)
-
-
-def generateDictOperationUpdate3Code(to_name, expression, emit, context):
-    dict_name, iterable_name, pair_names = generateChildExpressionsCode(
-        expression=expression, emit=emit, context=context
-    )
-
+def _generateDictOperationUpdateCommonCode(
+    dict_name, iterable_name, release_dict, expression, emit, context
+):
     res_name = context.getIntResName()
 
     emit("assert(PyDict_Check(%s));" % dict_name)
 
-    if expression.subnode_iterable is not None:
-        emit(
-            renderTemplateFromString(
-                r"""
+    emit(
+        renderTemplateFromString(
+            r"""
 {% if has_keys_attribute == None %}
 if (HAS_ATTR_BOOL(%(iterable_name)s, const_str_plain_keys)){
     %(res_name)s = PyDict_Merge(%(dict_name)s, %(iterable_name)s, 1);
@@ -449,42 +426,105 @@ if (HAS_ATTR_BOOL(%(iterable_name)s, const_str_plain_keys)){
     %(res_name)s = PyDict_MergeFromSeq2(%(dict_name)s, %(iterable_name)s, 1);
 {% endif %}
 """,
-                has_keys_attribute=expression.subnode_iterable.isKnownToHaveAttribute(
-                    "keys"
-                ),
-            )
-            % {
-                "res_name": res_name,
-                "dict_name": dict_name,
-                "iterable_name": iterable_name,
-            }
+            has_keys_attribute=expression.subnode_iterable.isKnownToHaveAttribute(
+                "keys"
+            ),
+        )
+        % {
+            "res_name": res_name,
+            "dict_name": dict_name,
+            "iterable_name": iterable_name,
+        }
+    )
+
+    if release_dict:
+        release_names = (dict_name, iterable_name)
+    else:
+        release_names = (iterable_name,)
+
+    getErrorExitBoolCode(
+        condition="%s != 0" % res_name,
+        release_names=release_names,
+        needs_check=expression.mayRaiseException(BaseException),
+        emit=emit,
+        context=context,
+    )
+
+
+def generateDictOperationUpdate2Code(to_name, expression, emit, context):
+    dict_name, iterable_name = generateChildExpressionsCode(
+        expression=expression, emit=emit, context=context
+    )
+
+    _generateDictOperationUpdateCommonCode(
+        dict_name=dict_name,
+        iterable_name=iterable_name,
+        release_dict=True,
+        expression=expression,
+        emit=emit,
+        context=context,
+    )
+
+    assignConstantNoneResult(to_name, emit, context)
+
+
+def generateDictOperationUpdate3Code(to_name, expression, emit, context):
+    dict_name = generateChildExpressionCode(
+        expression=expression.subnode_dict_arg, emit=emit, context=context
+    )
+
+    emit("assert(PyDict_Check(%s));" % dict_name)
+
+    if expression.subnode_iterable is not None:
+        iterable_name = generateChildExpressionCode(
+            expression=expression.subnode_iterable, emit=emit, context=context
         )
 
-        getErrorExitBoolCode(
-            condition="%s != 0" % res_name,
-            needs_check=expression.mayRaiseException(BaseException),
+        _generateDictOperationUpdateCommonCode(
+            dict_name=dict_name,
+            iterable_name=iterable_name,
+            release_dict=False,
+            expression=expression,
             emit=emit,
             context=context,
         )
 
-    for count, (pair_key_name, pair_key_value) in enumerate(pair_names):
-        if python_version < 0x350:
-            pair_key_name, pair_key_value = pair_key_value, pair_key_name
+    # Unique values per build here, but nested.
+    dict_key_name = context.allocateTempName("dictupdate_key")
+    dict_value_name = context.allocateTempName("dictupdate_value")
+
+    res_name = context.getIntResName()
+
+    for count, pair in enumerate(expression.subnode_pairs):
+        generateExpressionCode(
+            to_name=dict_key_name,
+            expression=pair.getKeyNode(),
+            emit=emit,
+            context=context,
+        )
+
+        generateExpressionCode(
+            to_name=dict_value_name,
+            expression=pair.getValueNode(),
+            emit=emit,
+            context=context,
+        )
 
         emit(
             "%s = PyDict_SetItem(%s, %s, %s);"
-            % (res_name, dict_name, pair_key_name, pair_key_value)
+            % (res_name, dict_name, dict_key_name, dict_value_name)
         )
 
         getErrorExitBoolCode(
             condition="%s != 0" % res_name,
             needs_check=not expression.subnode_pairs[count].isKnownToBeHashable(),
+            release_names=(dict_key_name, dict_value_name),
             emit=emit,
             context=context,
         )
 
-    getReleaseCodes(
-        release_names=(dict_name, iterable_name) + sum(pair_names, ()),
+    getReleaseCode(
+        release_name=dict_name,
         emit=emit,
         context=context,
     )
@@ -697,8 +737,11 @@ def generateDictOperationSetCode(statement, emit, context):
     res_name = context.getIntResName()
 
     emit(
-        "%s = PyDict_SetItem(%s, %s, %s);"
-        % (res_name, dict_arg_name, key_arg_name, value_arg_name)
+        """\
+assert(PyDict_CheckExact(%s));
+%s = PyDict_SetItem(%s, %s, %s);
+"""
+        % (dict_arg_name, res_name, dict_arg_name, key_arg_name, value_arg_name)
     )
 
     getErrorExitBoolCode(
@@ -740,8 +783,11 @@ def generateDictOperationSetCodeKeyValue(statement, emit, context):
     res_name = context.getIntResName()
 
     emit(
-        "%s = PyDict_SetItem(%s, %s, %s);"
-        % (res_name, dict_arg_name, key_arg_name, value_arg_name)
+        """\
+assert(PyDict_CheckExact(%s));
+%s = PyDict_SetItem(%s, %s, %s);
+"""
+        % (dict_arg_name, res_name, dict_arg_name, key_arg_name, value_arg_name)
     )
 
     getErrorExitBoolCode(
@@ -772,7 +818,7 @@ def generateDictOperationRemoveCode(statement, emit, context):
 
     with context.withCurrentSourceCodeReference(
         statement.subnode_key.getSourceReference()
-        if Options.is_fullcompat
+        if Options.is_full_compat
         else statement.getSourceReference()
     ):
         res_name = context.getBoolResName()
