@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,7 +18,6 @@
 """ Eval/exec/execfile/compile built-in related codes. """
 
 from nuitka import Options
-from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
 from nuitka.PythonVersions import python_version
 
 from .CodeHelpers import (
@@ -266,7 +265,7 @@ def generateExecCode(statement, emit, context):
     filename_name.getCType().emitAssignmentCodeFromConstant(
         to_name=filename_name,
         constant="<string>"
-        if Options.is_fullcompat
+        if Options.is_full_compat
         else "<string at %s>" % source_ref.getAsString(),
         may_escape=False,
         emit=emit,
@@ -281,7 +280,7 @@ def generateExecCode(statement, emit, context):
 
     with context.withCurrentSourceCodeReference(
         locals_arg.getSourceReference()
-        if Options.is_fullcompat
+        if Options.is_full_compat
         else statement.getSourceReference()
     ):
 
@@ -412,7 +411,7 @@ def generateLocalsDictSyncCode(statement, emit, context):
         _getStoreLocalsCode(
             locals_name=locals_name,
             variable_traces=statement.getPreviousVariablesTraces(),
-            is_dict=locals_arg.getTypeShape() is tshape_dict,
+            is_dict=locals_arg.hasShapeDictionaryExact(),
             emit=emit,
             context=context,
         )
