@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -128,7 +128,7 @@ def getModuleNameAndKindFromFilename(module_filename):
                 module_name = ModuleName(
                     os.path.basename(module_filename)[: -len(suffix)]
                 )
-                module_kind = "shlib"
+                module_kind = "extension"
 
                 break
         else:
@@ -213,7 +213,7 @@ def findModule(module_name, parent_package, level):
         method used.
     """
     # We have many branches here, because there are a lot of cases to try.
-    # pylint: disable=too-many-branches,too-many-return-statements,too-many-statements
+    # pylint: disable=too-many-branches,too-many-return-statements
 
     assert type(module_name) is ModuleName, module_name
 
@@ -227,8 +227,6 @@ def findModule(module_name, parent_package, level):
     # that name, but it would be wasteful.
     assert module_name != "*"
 
-    tried_names = []
-
     if level > 1:
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
@@ -245,7 +243,6 @@ def findModule(module_name, parent_package, level):
             full_name = ModuleName(parent_package)
 
         full_name = normalizePackageName(full_name)
-        tried_names.append(full_name)
 
         preloaded_path = getPreloadedPackagePath(module_name)
 
@@ -272,9 +269,8 @@ def findModule(module_name, parent_package, level):
 
             return full_name.getPackageName(), module_filename, "relative"
 
-    if level <= 1 and module_name != "":
+    if level < 1 and module_name != "":
         module_name = normalizePackageName(module_name)
-        tried_names.append(module_name)
 
         package_name = module_name.getPackageName()
 
@@ -354,7 +350,7 @@ def _reportCandidates(package_name, module_name, candidate, candidates):
 
     if (
         candidate.priority == 1
-        and Options.shallPreferSourcecodeOverExtensionModules() is None
+        and Options.shallPreferSourceCodeOverExtensionModules() is None
     ):
         for c in candidates:
             # Don't compare to itself and don't consider unused bytecode a problem.
@@ -391,7 +387,7 @@ def _findModuleInPath2(package_name, module_name, search_path):
     # Higher values are lower priority.
     priority_map = {
         imp.PY_COMPILED: 3,
-        imp.PY_SOURCE: 0 if Options.shallPreferSourcecodeOverExtensionModules() else 2,
+        imp.PY_SOURCE: 0 if Options.shallPreferSourceCodeOverExtensionModules() else 2,
         imp.C_EXTENSION: 1,
     }
 
@@ -409,8 +405,8 @@ def _findModuleInPath2(package_name, module_name, search_path):
         if os.path.isdir(package_directory):
             found = False
 
-            for suffix, _mode, mtype in imp.get_suffixes():
-                if mtype == imp.C_EXTENSION:
+            for suffix, _mode, module_type in imp.get_suffixes():
+                if module_type == imp.C_EXTENSION:
                     continue
 
                 package_file_name = "__init__" + suffix
@@ -421,7 +417,7 @@ def _findModuleInPath2(package_name, module_name, search_path):
                     candidates.add(
                         ImportScanFinding(
                             found_in=entry,
-                            priority=priority_map[mtype],
+                            priority=priority_map[module_type],
                             full_path=package_directory,
                             search_order=count,
                         )
@@ -439,10 +435,10 @@ def _findModuleInPath2(package_name, module_name, search_path):
                 )
 
         # Then, check out suffixes of all kinds, but only for one directory.
-        last_mtype = 0
-        for suffix, _mode, mtype in imp.get_suffixes():
+        last_module_type = 0
+        for suffix, _mode, module_type in imp.get_suffixes():
             # Use first match per kind only.
-            if mtype == last_mtype:
+            if module_type == last_module_type:
                 continue
 
             full_path = os.path.join(entry, module_name + suffix)
@@ -451,12 +447,12 @@ def _findModuleInPath2(package_name, module_name, search_path):
                 candidates.add(
                     ImportScanFinding(
                         found_in=entry,
-                        priority=4 + priority_map[mtype],
+                        priority=4 + priority_map[module_type],
                         full_path=full_path,
                         search_order=count,
                     )
                 )
-                last_mtype = mtype
+                last_module_type = module_type
 
     if _debug_module_finding:
         my_print("Candidates:", candidates)
@@ -528,7 +524,7 @@ def getPackageSearchPath(package_name):
     assert main_path is not None
 
     if package_name is None:
-        return [os.getcwd(), main_path] + [
+        result = [os.getcwd(), main_path] + [
             _unpackPathElement(path_element) for path_element in sys.path
         ]
     elif "." in package_name:
@@ -567,7 +563,8 @@ def getPackageSearchPath(package_name):
                 if isPackageDir(package_dir) or force_package:
                     result.append(package_dir)
 
-        return result
+    result = [element for element in result if os.path.exists(element)]
+    return OrderedSet(result)
 
 
 def _findModuleInPath(module_name):
@@ -698,7 +695,7 @@ def locateModules(package_name):
 def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     # Many branches due to the many cases
 
-    assert type(module_name) is ModuleName
+    assert type(module_name) is ModuleName, module_name
     assert filename is not None
 
     is_namespace = False
