@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -49,6 +49,10 @@ class ShapeBase(getMetaClassBase("Shape")):
     def getShapeIter():
         return tshape_unknown
 
+    @staticmethod
+    def hasShapeIndexLookup():
+        return None
+
     @staticmethod
     def hasShapeModule():
         return None
@@ -93,6 +97,10 @@ class ShapeBase(getMetaClassBase("Shape")):
     def hasShapeSlotNext():
         return None
 
+    @staticmethod
+    def hasShapeSlotNextCode():
+        return None
+
     @staticmethod
     def hasShapeSlotContains():
         return None
@@ -105,6 +113,10 @@ class ShapeBase(getMetaClassBase("Shape")):
     def hasShapeTrustedAttributes():
         return None
 
+    @staticmethod
+    def isShapeIterator():
+        return None
+
     add_shapes = {}
 
     def getOperationBinaryAddShape(self, right_shape):
@@ -468,6 +480,12 @@ class ShapeBase(getMetaClassBase("Shape")):
     def getOperationUnaryReprEscape(self):
         pass
 
+    @staticmethod
+    def isKnownToHaveAttribute(attribute_name):
+        # Virtual method, pylint: disable=unused-argument
+
+        return None
+
     def emitAlternatives(self, emit):
         emit(self)
 
@@ -545,11 +563,11 @@ class ShapeTypeUnknown(ShapeBase):
 tshape_unknown = ShapeTypeUnknown()
 
 
-class ShapeTypeUninit(ShapeTypeUnknown):
+class ShapeTypeUninitialized(ShapeTypeUnknown):
     pass
 
 
-tshape_uninit = ShapeTypeUninit()
+tshape_uninitialized = ShapeTypeUninitialized()
 
 
 class ValueShapeBase(object):
@@ -606,6 +624,10 @@ class ShapeLargeConstantValuePredictable(ShapeLargeConstantValue):
 class ShapeIterator(ShapeBase, ShapeIteratorMixin):
     """Iterator created by iter with 2 arguments, TODO: could be way more specific."""
 
+    @staticmethod
+    def isShapeIterator():
+        return None
+
     @staticmethod
     def hasShapeSlotBool():
         return None
