@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,7 +33,7 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
 from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
-from nuitka.nodes.shapes.StandardShapes import tshape_uninit
+from nuitka.nodes.shapes.StandardShapes import tshape_uninitialized
 from nuitka.tree.SourceReading import readSourceLine
 from nuitka.utils.InstanceCounters import (
     counted_del,
@@ -44,6 +44,8 @@ from nuitka.utils.Timing import TimerReport
 
 from .ValueTraces import (
     ValueTraceAssign,
+    ValueTraceAssignUnescapable,
+    ValueTraceAssignUnescapablePropagated,
     ValueTraceDeleted,
     ValueTraceEscaped,
     ValueTraceInit,
@@ -52,7 +54,7 @@ from .ValueTraces import (
     ValueTraceLoopComplete,
     ValueTraceLoopIncomplete,
     ValueTraceMerge,
-    ValueTraceUninit,
+    ValueTraceUninitialized,
     ValueTraceUnknown,
 )
 
@@ -254,7 +256,7 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
         return trace
 
     def _initVariableUninit(self, variable):
-        trace = ValueTraceUninit(owner=self.owner, previous=None)
+        trace = ValueTraceUninitialized(owner=self.owner, previous=None)
 
         self.addVariableTrace(variable, 0, trace)
 
@@ -332,6 +334,25 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
 
             self.markActiveVariableAsEscaped(variable)
 
+    def onUsedFunction(self, function_body):
+        owning_module = function_body.getParentModule()
+
+        # Make sure the owning module is added to the used set. This is most
+        # important for helper functions, or modules, which otherwise have
+        # become unused.
+        addUsedModule(
+            module=owning_module,
+            using_module=None,
+            usage_tag="function",
+            reason="Function %s" % self.name,
+            source_ref=owning_module.source_ref,
+        )
+
+        needs_visit = owning_module.addUsedFunction(function_body)
+
+        if needs_visit:
+            function_body.computeFunctionRaw(self)
+
 
 class TraceCollectionBase(object):
     """This contains for logic for maintaining active traces.
@@ -339,7 +360,7 @@ class TraceCollectionBase(object):
     They are kept for "variable" and versions.
     """
 
-    __slots__ = ("owner", "parent", "name", "value_states", "variable_actives")
+    __slots__ = ("owner", "parent", "name", "variable_actives")
 
     if isCountingInstances():
         __del__ = counted_del()
@@ -350,9 +371,6 @@ class TraceCollectionBase(object):
         self.parent = parent
         self.name = name
 
-        # Value state extra information per node.
-        self.value_states = {}
-
         # Currently active values in the tracing.
         self.variable_actives = {}
 
@@ -394,7 +412,7 @@ class TraceCollectionBase(object):
     def markActiveVariableAsEscaped(self, variable):
         current = self.getVariableCurrentTrace(variable)
 
-        if not current.isEscapeOrUnknownOrUninitTrace():
+        if current.isTraceThatNeedsEscape():
             version = variable.allocateTargetNumber()
 
             self.addVariableTrace(
@@ -430,7 +448,7 @@ class TraceCollectionBase(object):
             # assert shapes, (variable, current)
 
             if not shapes:
-                shapes.add(tshape_uninit)
+                shapes.add(tshape_uninitialized)
 
             result = ValueTraceLoopComplete(loop_node, current, shapes)
 
@@ -460,25 +478,6 @@ class TraceCollectionBase(object):
         # This is monkey patched from another module. pylint: disable=I0021,not-callable
         signalChange(tags, source_ref, message)
 
-    def onUsedFunction(self, function_body):
-        owning_module = function_body.getParentModule()
-
-        # Make sure the owning module is added to the used set. This is most
-        # important for helper functions, or modules, which otherwise have
-        # become unused.
-        addUsedModule(
-            module=owning_module,
-            using_module=None,
-            usage_tag="function",
-            reason="Function %s" % self.name,
-            source_ref=owning_module.source_ref,
-        )
-
-        needs_visit = owning_module.addUsedFunction(function_body)
-
-        if needs_visit:
-            function_body.computeFunctionRaw(self)
-
     @staticmethod
     def mustAlias(a, b):
         if a.isExpressionVariableRef() and b.isExpressionVariableRef():
@@ -542,6 +541,37 @@ class TraceCollectionBase(object):
 
         return variable_trace
 
+    def onVariableSetToUnescapableValue(self, variable, version, assign_node):
+        variable_trace = ValueTraceAssignUnescapable(
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable),
+        )
+
+        self.addVariableTrace(variable, version, variable_trace)
+
+        # Make references point to it.
+        self.markCurrentVariableTrace(variable, version)
+
+        return variable_trace
+
+    def onVariableSetToUnescapablePropagatedValue(
+        self, variable, version, assign_node, replacement
+    ):
+        variable_trace = ValueTraceAssignUnescapablePropagated(
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable),
+            replacement=replacement,
+        )
+
+        self.addVariableTrace(variable, version, variable_trace)
+
+        # Make references point to it.
+        self.markCurrentVariableTrace(variable, version)
+
+        return variable_trace
+
     def onVariableDel(self, variable, version, del_node):
         # Add a new trace, allocating a new version for the variable, and
         # remember the delete of the current
@@ -907,27 +937,15 @@ class TraceCollectionBase(object):
 
         return new_node, change_tags, message
 
-    def getIteratorNextCount(self, iter_node):
-        return self.value_states.get(iter_node)
-
-    def initIteratorValue(self, iter_node):
-        # TODO: More complex state information will be needed eventually.
-        self.value_states[iter_node] = 0
-
-    def onIteratorNext(self, iter_node):
-        if iter_node in self.value_states:
-            self.value_states[iter_node] += 1
-
-    def resetValueStates(self):
-        for key in self.value_states:
-            self.value_states[key] = None
-
     def addOutlineFunction(self, outline):
         self.parent.addOutlineFunction(outline)
 
     def getVeryTrustedModuleVariables(self):
         return self.parent.getVeryTrustedModuleVariables()
 
+    def onUsedFunction(self, function_body):
+        return self.parent.onUsedFunction(function_body)
+
 
 class TraceCollectionBranch(CollectionUpdateMixin, TraceCollectionBase):
     __slots__ = ("variable_traces",)
@@ -1069,6 +1087,8 @@ class TraceCollectionPureFunction(TraceCollectionFunction):
     def onUsedFunction(self, function_body):
         self.used_functions.add(function_body)
 
+        TraceCollectionFunction.onUsedFunction(self, function_body=function_body)
+
 
 class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
     __slots__ = (
@@ -1120,7 +1140,7 @@ def areEmptyTraces(variable_traces):
             # from being removed.
 
             return False
-        elif variable_trace.isUninitTrace():
+        elif variable_trace.isUninitializedTrace():
             if variable_trace.getUsageCount():
                 # Checking definite is enough, the merges, we shall see
                 # them as well.
@@ -1162,7 +1182,7 @@ def areReadOnlyTraces(variable_traces):
             # from being not released.
 
             return False
-        elif variable_trace.isUninitTrace():
+        elif variable_trace.isUninitializedTrace():
             pass
         elif variable_trace.isUnknownTrace():
             return False
