@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,25 +23,27 @@ to add to this and submit patches to make it more complete.
 """
 
 import fnmatch
-import os
-import sys
 
 from nuitka.__past__ import iter_modules
-from nuitka.containers.oset import OrderedSet
+from nuitka.Options import (
+    isOnefileMode,
+    isStandaloneMode,
+    mayDisableConsoleWindow,
+    shallCreateAppBundle,
+    shallDisableConsoleWindow,
+)
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
-from nuitka.utils.FileOperations import getFileContentByLine
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.SharedLibraries import getPyWin32Dir
-from nuitka.utils.Utils import getOS, isLinux, isMacOS, isWin32Windows
-from nuitka.utils.Yaml import parsePackageYaml
+from nuitka.utils.Utils import getOS, isMacOS, isWin32Windows
+from nuitka.utils.Yaml import getYamlPackageConfiguration
 
 
-class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
+class NuitkaPluginImplicitImports(NuitkaPluginBase):
     plugin_name = "implicit-imports"
 
     def __init__(self):
-        self.config = parsePackageYaml(__package__, "implicit-imports.yml")
+        self.config = getYamlPackageConfiguration()
 
     @staticmethod
     def isAlwaysEnabled():
@@ -58,10 +60,11 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                     "Error, invalid pattern with empty parts used '%s'." % pattern
                 )
 
-            if "." in part or "*" in part:
+            # TODO: Checking for shell pattern should be done in more places and shared code.
+            if "?" in part or "*" in part or "[" in part:
                 if current is None:
                     self.sysexit(
-                        "Error, cannot use patter for first part '%s'." % pattern
+                        "Error, cannot use pattern for first part '%s'." % pattern
                     )
 
                 module_filename = self.locateModule(
@@ -91,23 +94,99 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
 
         yield current
 
-    def _getImportsByFullname(self, full_name):
-        """Provides names of modules to imported implicitly.
-
-        Notes:
-            This methods works much like 'getImplicitImports', except that it
-            accepts the search argument as a string. This allows callers to
-            obtain results, which cannot provide a Nuitka module object.
-        """
+    def _getImportsByFullname(self, module, full_name):
+        """Provides names of modules to imported implicitly."""
         # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
 
-        config = self.config.get(full_name)
+        config = self.config.get(full_name, section="implicit-imports")
 
         # Checking for config, but also allowing fall through.
         if config:
-            dependencies = config.get("depends")
+            dependencies = config.get("depends", [])
+            standalone_macos_bundle_mode = config.get("standalone_macos_bundle_mode")
+            disable_console = config.get("disable_console")
 
-            if type(dependencies) is not list or not dependencies:
+            if standalone_macos_bundle_mode is not None:
+                if standalone_macos_bundle_mode is True:
+                    standalone_macos_bundle_mode = "yes"
+                if standalone_macos_bundle_mode is False:
+                    standalone_macos_bundle_mode = "no"
+
+                if type(standalone_macos_bundle_mode) is not str:
+                    self.sysexit(
+                        "Error, requiring str as 'standalone_macos_bundle_mode' value for '%s' entry."
+                        % full_name
+                    )
+
+                if standalone_macos_bundle_mode not in ("yes", "no", "onefile"):
+                    self.sysexit(
+                        "Error, illegal value '%s' for 'standalone_macos_bundle_mode' for '%s' entry."
+                        % (standalone_macos_bundle_mode, full_name)
+                    )
+
+                if isMacOS():
+                    if standalone_macos_bundle_mode == "yes":
+                        if isStandaloneMode() and not shallCreateAppBundle():
+                            self.sysexit(
+                                """\
+Error, package '%s' requires '--macos-create-app-bundle' to be used or else it cannot work."""
+                                % full_name
+                            )
+                    elif standalone_macos_bundle_mode == "onefile":
+                        if shallCreateAppBundle() and not isOnefileMode():
+                            self.sysexit(
+                                """\
+Error, package '%s' requires '--onefile' to be used on top of '--macos-create-app-bundle' or else it cannot work."""
+                                % full_name
+                            )
+
+            if disable_console is not None:
+                if disable_console is True:
+                    disable_console = "yes"
+                if disable_console is False:
+                    disable_console = "no"
+
+                if type(disable_console) is not str:
+                    self.sysexit(
+                        "Error, requiring str as 'disable_console' value for '%s' entry."
+                        % full_name
+                    )
+
+                # Required on macOS is recommended elsewhere.
+                if not isMacOS() and disable_console == "recommend_on_macos":
+                    disable_console = "recommend"
+
+                if (
+                    disable_console == "recommend"
+                    and mayDisableConsoleWindow()
+                    and shallDisableConsoleWindow() is None
+                ):
+                    if isMacOS():
+                        downside_message = "Otherwise high resolution will not be available and a terminal window will open."
+                    else:
+                        downside_message = "Otherwise a terminal window will open."
+
+                    self.info(
+                        "Note, when using '%s', consider using '--disable-console' option. %s"
+                        % (full_name, downside_message)
+                    )
+
+                if (
+                    disable_console == "yes"
+                    and mayDisableConsoleWindow()
+                    and shallDisableConsoleWindow() is not True
+                ):
+                    self.sysexit(
+                        "Note, when using '%s', you have to use '--disable-console' option."
+                        % full_name
+                    )
+
+            if (
+                type(dependencies) is not list
+                and not dependencies
+                and standalone_macos_bundle_mode is None
+                and disable_console is None
+            ):
                 self.sysexit(
                     "Error, requiring list below 'depends' entry for '%s' entry."
                     % full_name
@@ -115,7 +194,15 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
 
             for dependency in dependencies:
                 if dependency.startswith("."):
-                    dependency = full_name.getChildNamed(dependency[1:]).asString()
+                    if (
+                        module.isUncompiledPythonPackage()
+                        or module.isCompiledPythonPackage()
+                    ):
+                        dependency = full_name.getChildNamed(dependency[1:]).asString()
+                    else:
+                        dependency = full_name.getSiblingNamed(
+                            dependency[1:]
+                        ).asString()
 
                 if "*" in dependency or "?" in dependency:
                     for resolved in self._resolveModulePattern(dependency):
@@ -704,7 +791,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "pywt._extensions._swt"
 
         # imageio imports -----------------------------------------------
-        elif full_name == "imageio":
+        elif full_name == "PIL.Image":
             yield "PIL.BlpImagePlugin"
             yield "PIL.BmpImagePlugin"
             yield "PIL.BufrStubImagePlugin"
@@ -751,6 +838,21 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "PIL.XpmImagePlugin"
             yield "PIL.XVThumbImagePlugin"
 
+        # rapidfuzz imports -----------------------------------------------
+        elif full_name == "rapidfuzz":
+            yield "rapidfuzz.utils_cpp"
+            yield "rapidfuzz.fuzz_cpp"
+            yield "rapidfuzz.process_cdist_cpp"
+            yield "rapidfuzz.process_cpp"
+            yield "rapidfuzz.string_metric_cpp"
+
+        elif full_name == "rapidfuzz.distance":
+            yield "rapidfuzz.distance._initialize_cpp"
+            yield "rapidfuzz.distance.Hamming_cpp"
+            yield "rapidfuzz.distance.Indel_cpp"
+            yield "rapidfuzz.distance.LCSseq_cpp"
+            yield "rapidfuzz.distance.Levenshtein_cpp"
+
         # scikit-image imports -----------------------------------------------
         elif full_name == "skimage.draw":
             yield "skimage.draw._draw"
@@ -929,6 +1031,15 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "sklearn.utils.weight_vector"
             yield "sklearn.utils._cython_blas"
             yield "sklearn.utils._logistic_sigmoid"
+            yield "sklearn.utils._weight_vector"
+            yield "sklearn.utils._typedefs"
+            yield "sklearn.utils._heap"
+            yield "sklearn.utils._sorting"
+            yield "sklearn.utils._vector_sentinel"
+            yield "sklearn.utils._seq_dataset"
+            yield "sklearn.utils._readonly_array_wrapper"
+            yield "sklearn.utils._openmp_helpers"
+            yield "sklearn.utils._fast_dict"
             yield "sklearn.utils._random"
 
         elif full_name == "sklearn.utils.sparsetools":
@@ -947,6 +1058,11 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "pkg_resources._vendor.packaging.specifiers"
             yield "pkg_resources._vendor.packaging.requirements"
 
+        elif full_name == "pkg_resources._vendor.jaraco":
+            yield "pkg_resources._vendor.jaraco.text"
+            yield "pkg_resources._vendor.jaraco.functools"
+            yield "pkg_resources._vendor.jaraco.context"
+
         # TODO: Is this even true, or an artifact of how we handled requests.packages.urllib3 in the past.
         # urllib3 -------------------------------------------------------------
         elif full_name in ("urllib3", "requests_toolbelt._compat"):
@@ -1076,6 +1192,8 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 yield crypto_module_name + ".PublicKey._ec_ws"
             elif full_name == crypto_module_name + ".Cipher.ARC4":
                 yield crypto_module_name + ".Cipher._ARC4"
+            elif full_name == crypto_module_name + ".Cipher.PKCS1_v1_5":
+                yield crypto_module_name + ".Cipher._pkcs1_decode"
             elif full_name == crypto_module_name + ".Math._IntegerCustom":
                 yield crypto_module_name + ".Math._modexp"
         elif full_name == "pycparser.c_parser":
@@ -1108,7 +1226,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             elif isWin32Windows():
                 yield full_name.getChildNamed("_win32")
             else:
-                yield full_name.getChildNamed("xorg")
+                yield full_name.getChildNamed("_xorg")
         elif full_name == "_pytest._code.code":
             yield "py._path.local"
         elif full_name == "pyreadstat._readstat_parser":
@@ -1116,6 +1234,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         elif full_name == "pyreadstat.pyreadstat":
             yield "pyreadstat._readstat_writer"
             yield "pyreadstat.worker"
+            yield "multiprocessing"
         elif full_name == "cytoolz.itertoolz":
             yield "cytoolz.utils"
         elif full_name == "cytoolz.functoolz":
@@ -1146,48 +1265,32 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "mercurial.charencode"
             yield "mercurial.cext.parsers"
 
-    def getImportsByFullname(self, full_name):
-        """Recursively create a set of imports for a fullname.
-
-        Notes:
-            If an imported item has imported kids, call again with each new item,
-            resulting in a leaf-only set (no more consequential kids).
-        """
-        result = OrderedSet()
-
-        def checkImportsRecursive(module_name):
-            for item in self._getImportsByFullname(module_name):
-                item = ModuleName(item)
-
-                if item not in result:
-                    result.add(item)
-                    checkImportsRecursive(item)
-
-        checkImportsRecursive(full_name)
-
-        if full_name in result:
-            result.remove(full_name)
-
-        return result
-
     def getImplicitImports(self, module):
         full_name = module.getFullName()
 
-        if module.isPythonShlibModule():
+        # TODO: This code absolutely doesn't below here.
+        if module.isPythonExtensionModule():
             for used_module in module.getUsedModules():
                 yield used_module[0]
 
         if full_name == "pkg_resources.extern":
-            for line in getFileContentByLine(module.getCompileTimeFilename()):
-                if line.startswith("names"):
-                    line = line.split("=")[-1].strip()
-                    parts = line.split(",")
+            # TODO: A package specific lookup of compile time "pkg_resources.extern" could
+            # be done here, but this might be simpler to hardcode for now. Once we have
+            # the infrastructure to ask a module that after optimization, we should do
+            # that instead, as it will not use a separate process.
+            for part in (
+                "packaging",
+                "pyparsing",
+                "appdirs",
+                "jaraco",
+                "importlib_resources",
+                "more_itertools",
+                "six",
+            ):
+                yield "pkg_resources._vendor." + part
 
-                    for part in parts:
-                        yield "pkg_resources._vendor." + part.strip("' ")
         else:
-            # create a flattened import set for full_name and yield from it
-            for item in self.getImportsByFullname(full_name):
+            for item in self._getImportsByFullname(module=module, full_name=full_name):
                 yield item
 
     def onModuleSourceCode(self, module_name, source_code):
@@ -1205,82 +1308,6 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         # Do nothing by default.
         return source_code
 
-    def getExtraDlls(self, module):
-        full_name = module.getFullName()
-
-        if full_name == "uuid" and isLinux():
-            uuid_dll_path = self.locateDLL("uuid")
-
-            if uuid_dll_path is not None:
-                yield self.makeDllEntryPoint(
-                    uuid_dll_path, os.path.basename(uuid_dll_path), None
-                )
-        elif full_name == "iptc" and isLinux():
-            import iptc.util  # pylint: disable=I0021,import-error
-
-            xtwrapper_dll = iptc.util.find_library("xtwrapper")[0]
-            xtwrapper_dll_path = xtwrapper_dll._name  # pylint: disable=protected-access
-
-            yield self.makeDllEntryPoint(
-                xtwrapper_dll_path, os.path.basename(xtwrapper_dll_path), None
-            )
-        elif full_name == "coincurve._libsecp256k1" and isWin32Windows():
-            yield self.makeDllEntryPoint(
-                os.path.join(module.getCompileTimeDirectory(), "libsecp256k1.dll"),
-                os.path.join(full_name.getPackageName(), "libsecp256k1.dll"),
-                full_name.getPackageName(),
-            )
-        # TODO: This should be its own plugin.
-        elif (
-            full_name
-            in (
-                "pythoncom",
-                "win32api",
-                "win32clipboard",
-                "win32console",
-                "win32cred",
-                "win32crypt",
-                "win32event",
-                "win32evtlog",
-                "win32file",
-                "win32gui",
-                "win32help",
-                "win32inet",
-                "win32job",
-                "win32lz",
-                "win32net",
-                "win32pdh",
-                "win32pipe",
-                "win32print",
-                "win32process",
-                "win32profile",
-                "win32ras",
-                "win32security",
-                "win32service",
-                "win32trace",
-                "win32transaction",
-                "win32ts",
-                "win32wnet",
-            )
-            and isWin32Windows()
-        ):
-            pywin_dir = getPyWin32Dir()
-
-            if pywin_dir is not None:
-                for dll_name in "pythoncom", "pywintypes":
-
-                    pythoncom_filename = "%s%d%d.dll" % (
-                        dll_name,
-                        sys.version_info[0],
-                        sys.version_info[1],
-                    )
-                    pythoncom_dll_path = os.path.join(pywin_dir, pythoncom_filename)
-
-                    if os.path.exists(pythoncom_dll_path):
-                        yield self.makeDllEntryPoint(
-                            pythoncom_dll_path, pythoncom_filename, None
-                        )
-
     unworthy_namespaces = (
         "setuptools",  # Not performance relevant.
         "distutils",  # Not performance relevant.
@@ -1308,8 +1335,12 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         "telethon.tl.types",  # Not performance relevant and slow C compile
         "importlib_metadata",  # Not performance relevant and slow C compile
         "comtypes.gen",  # Not performance relevant and slow C compile
+        "win32com.gen_py",  # Not performance relevant and slow C compile
         "phonenumbers.geodata",  # Not performance relevant and slow C compile
         "site",  # Not performance relevant and problems with .pth files
+        "packaging",  # Not performance relevant.
+        "appdirs",  # Not performance relevant.
+        "dropbox.team_log",  # Too large generated code
     )
 
     def decideCompilation(self, module_name):
