@@ -1,4 +1,4 @@
-#     Copyright 2021, Jorj McKie, mailto:<jorj.x.mckie@outlook.de>
+#     Copyright 2022, Jorj McKie, mailto:<jorj.x.mckie@outlook.de>
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,8 +21,9 @@ import os
 import sys
 
 from nuitka import Options
-from nuitka.freezer.IncludedDataFiles import makeIncludedDataDirectory
 from nuitka.plugins.PluginBase import NuitkaPluginBase
+from nuitka.utils.FileOperations import listDllFilesFromDirectory, relpath
+from nuitka.utils.Utils import isWin32Windows
 
 
 def _isTkInterModule(module):
@@ -116,6 +117,34 @@ override it here. Default is automatic detection.""",
 The Tcl library dir. See comments for Tk library dir.""",
         )
 
+    @staticmethod
+    def _getTkinterDnDPlatformDirectory():
+        # From their code:
+        import platform
+
+        if platform.system() == "Darwin":
+            return "osx64"
+        elif platform.system() == "Linux":
+            return "linux64"
+        elif platform.system() == "Windows":
+            return "win64"
+        else:
+            return None
+
+    def _considerDataFilesTkinterDnD(self, module):
+        platform_rep = self._getTkinterDnDPlatformDirectory()
+
+        if platform_rep is None:
+            return
+
+        yield self.makeIncludedPackageDataFiles(
+            package_name="tkinterdnd2",
+            package_directory=module.getCompileTimeDirectory(),
+            pattern=os.path.join("tkinterdnd2", "tkdnd", platform_rep, "**"),
+            reason="Tcl needed for 'tkinterdnd2' usage",
+            tags="tcl",
+        )
+
     def considerDataFiles(self, module):
         """Provide TCL libraries to the dist folder.
 
@@ -129,36 +158,43 @@ The Tcl library dir. See comments for Tk library dir.""",
         Yields:
             IncludedDataFile objects.
         """
+
+        # Extra TCL providing module go here:
+        if module.getFullName() == "tkinterdnd2.TkinterDnD":
+            yield self._considerDataFilesTkinterDnD(module)
+
+            return
+
         if not _isTkInterModule(module):
             return
 
         # Check typical locations of the dirs
         candidates_tcl = (
             os.environ.get("TCL_LIBRARY"),
-            os.path.join(sys.prefix, "tcl", "tcl8.5"),
-            os.path.join(sys.prefix, "tcl", "tcl8.6"),
-            os.path.join(sys.prefix, "lib", "tcl8.5"),
-            os.path.join(sys.prefix, "lib", "tcl8.6"),
             "/usr/share/tcltk/tcl8.6",
             "/usr/share/tcltk/tcl8.5",
             "/usr/share/tcl8.6",
             "/usr/share/tcl8.5",
             "/usr/lib64/tcl/tcl8.5",
             "/usr/lib64/tcl/tcl8.6",
+            os.path.join(sys.prefix, "tcl", "tcl8.5"),
+            os.path.join(sys.prefix, "tcl", "tcl8.6"),
+            os.path.join(sys.prefix, "lib", "tcl8.5"),
+            os.path.join(sys.prefix, "lib", "tcl8.6"),
         )
 
         candidates_tk = (
             os.environ.get("TK_LIBRARY"),
-            os.path.join(sys.prefix, "tcl", "tk8.5"),
-            os.path.join(sys.prefix, "tcl", "tk8.6"),
-            os.path.join(sys.prefix, "lib", "tk8.5"),
-            os.path.join(sys.prefix, "lib", "tk8.6"),
             "/usr/share/tcltk/tk8.6",
             "/usr/share/tcltk/tk8.5",
             "/usr/share/tk8.6",
             "/usr/share/tk8.5",
             "/usr/lib64/tcl/tk8.5",
             "/usr/lib64/tcl/tk8.6",
+            os.path.join(sys.prefix, "tcl", "tk8.5"),
+            os.path.join(sys.prefix, "tcl", "tk8.6"),
+            os.path.join(sys.prefix, "lib", "tk8.5"),
+            os.path.join(sys.prefix, "lib", "tk8.6"),
         )
 
         tcl = self.tcl_library_dir
@@ -184,18 +220,48 @@ The Tcl library dir. See comments for Tk library dir.""",
             )
 
         # survived the above, now do provide the locations
-        yield makeIncludedDataDirectory(
+        yield self.makeIncludedDataDirectory(
             source_path=tk,
             dest_path="tk",
             reason="Tk copy needed for standalone Tcl",
             ignore_dirs=("demos",),
+            tags="tk",
         )
-        yield makeIncludedDataDirectory(
+        yield self.makeIncludedDataDirectory(
             source_path=tcl,
             dest_path="tcl",
             reason="Tcl needed for tkinter usage",
+            tags="tcl",
         )
 
+        if isWin32Windows():
+            yield self.makeIncludedDataDirectory(
+                source_path=os.path.join(tcl, "..", "tcl8"),
+                dest_path="tcl8",
+                reason="Tcl needed for tkinter usage",
+                tags="tcl",
+            )
+
+    def getExtraDlls(self, module):
+        if module.getFullName() == "tkinterdnd2.TkinterDnD":
+            platform_rep = self._getTkinterDnDPlatformDirectory()
+
+            if platform_rep is None:
+                return
+
+            module_directory = module.getCompileTimeDirectory()
+
+            for filename, _dll_filename in listDllFilesFromDirectory(
+                os.path.join(module_directory, "tkdnd", platform_rep)
+            ):
+                dest_path = relpath(filename, module_directory)
+                yield self.makeDllEntryPoint(
+                    source_path=filename,
+                    dest_path=os.path.join("tkinterdnd2", dest_path),
+                    package_name="tkinterdnd2"
+                    # reason="tkinterdnd2 package DLL",
+                )
+
 
 class NuitkaPluginDetectorTkinter(NuitkaPluginBase):
     """Used only if plugin is not activated.
