@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,17 +30,12 @@ from optparse import OptionParser
 from nuitka.freezer.Onefile import checkOnefileReadiness
 from nuitka.tools.Basics import goHome
 from nuitka.tools.testing.Common import (
+    getInstalledPythonVersion,
     getTempDir,
     my_print,
     withExtendedExtraOptions,
 )
-from nuitka.utils.Execution import (
-    check_call,
-    check_output,
-    getExecutablePath,
-    getNullOutput,
-    getPythonExePathWindows,
-)
+from nuitka.utils.Execution import check_call, check_output, getNullOutput
 from nuitka.utils.FileOperations import (
     getFileContents,
     openTextFile,
@@ -543,7 +538,7 @@ def main():
     # There are many cases to deal with,
     # pylint: disable=too-many-branches,too-many-statements
 
-    # Lets honor this Debian option here.
+    # Lets honor this Debian option here, spell-checker: ignore nocheck
     if "nocheck" in os.environ.get("DEB_BUILD_OPTIONS", "").split():
         my_print("Skipped all tests as per DEB_BUILD_OPTIONS environment.")
         sys.exit(0)
@@ -626,10 +621,13 @@ def main():
             if command.startswith("python"):
                 remainder = command[6:]
 
-                if len(remainder) == 3 and remainder[1] == ".":
-                    command = getPythonExePathWindows(search=remainder, arch=None)
-
-                    return True
+                if 3 <= len(remainder) <= 4 and remainder[1] == ".":
+                    return (
+                        getInstalledPythonVersion(
+                            python_version=remainder, must_exist=False
+                        )
+                        is not None
+                    )
 
         return False
 
@@ -691,12 +689,9 @@ def main():
         if sys.version.startswith(intended_version):
             os.environ["PYTHON"] = sys.executable
         else:
-            if os.name == "nt":
-                os.environ["PYTHON"] = getPythonExePathWindows(
-                    search=intended_version, arch=None
-                )
-            else:
-                os.environ["PYTHON"] = getExecutablePath(use_python)
+            os.environ["PYTHON"] = getInstalledPythonVersion(
+                intended_version, must_exist=True
+            ).getPythonExe()
 
         if options.basic_tests:
             my_print(
