@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,27 +22,31 @@ know the variables written by a piece of code ahead of abstractly executing a
 loop.
 """
 
+from nuitka.containers.oset import OrderedSet
+
 from .Operations import VisitorNoopMixin, visitTree
 
 
-class VariableWriteExtractor(VisitorNoopMixin):
-    """ Extract variables written to.
+class VariableUsageExtractor(VisitorNoopMixin):
+    """Extract variables used."""
 
-    """
     def __init__(self):
-        self.written_to = set()
+        self.written_to = OrderedSet()
 
     def onEnterNode(self, node):
-        if node.isStatementAssignmentVariable() or \
-           node.isStatementDelVariable():
+        if (
+            node.isStatementAssignmentVariable()
+            or node.isStatementDelVariable()
+            or node.isExpressionVariableRef()
+        ):
             self.written_to.add(node.getVariable())
 
     def getResult(self):
         return self.written_to
 
 
-def getVariablesWritten(node):
-    visitor = VariableWriteExtractor()
+def getVariablesWrittenOrRead(node):
+    visitor = VariableUsageExtractor()
     visitTree(node, visitor)
 
     return visitor.getResult()
@@ -54,17 +58,16 @@ class VariableUsageUpdater(VisitorNoopMixin):
         self.new_variable = new_variable
 
     def onEnterNode(self, node):
-        if node.isStatementAssignmentVariable() or \
-           node.isStatementDelVariable() or \
-           node.isStatementReleaseVariable():
+        if (
+            node.isStatementAssignmentVariable()
+            or node.isStatementDelVariable()
+            or node.isStatementReleaseVariable()
+        ):
             if node.getVariable() is self.old_variable:
                 node.setVariable(self.new_variable)
 
 
 def updateVariableUsage(provider, old_variable, new_variable):
-    visitor = VariableUsageUpdater(
-        old_variable = old_variable,
-        new_variable = new_variable
-    )
+    visitor = VariableUsageUpdater(old_variable=old_variable, new_variable=new_variable)
 
     visitTree(provider, visitor)
