@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,16 +17,11 @@
 #
 """ Reformulation of function statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AsyncgenNodes import (
     ExpressionAsyncgenObjectBody,
     ExpressionMakeAsyncgenObject,
@@ -55,11 +50,18 @@ from nuitka.nodes.GeneratorNodes import (
 from nuitka.nodes.LocalsDictNodes import StatementSetLocalsDictionary
 from nuitka.nodes.OutlineNodes import ExpressionOutlineFunction
 from nuitka.nodes.ReturnNodes import StatementReturn, StatementReturnNone
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import StatementReleaseVariable
+from nuitka.nodes.VariableNameNodes import (
+    ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
+)
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableNameRef,
     ExpressionVariableRef,
 )
+from nuitka.Options import hasPythonFlagNoAnnotations
+from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
@@ -133,6 +135,10 @@ def buildFunctionNode(provider, node, source_ref):
 
     assert getKind(node) == "FunctionDef"
 
+    Plugins.onFunctionBodyParsing(
+        provider=provider, function_name=node.name, body=node.body
+    )
+
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
     function_kind, flags = detectFunctionBodyKind(nodes=function_statement_nodes)
@@ -293,6 +299,10 @@ def buildAsyncFunctionNode(provider, node, source_ref):
     # many details each, pylint: disable=too-many-locals
     assert getKind(node) == "AsyncFunctionDef"
 
+    Plugins.onFunctionBodyParsing(
+        provider=provider, function_name=node.name, body=node.body
+    )
+
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
     function_kind, flags = detectFunctionBodyKind(
@@ -454,7 +464,7 @@ def buildParameterAnnotations(provider, node, source_ref):
     # Too many branches, because there is too many cases, pylint: disable=too-many-branches
 
     # Build annotations. We are hiding here, that it is a Python3 only feature.
-    if python_version < 0x300:
+    if python_version < 0x300 or hasPythonFlagNoAnnotations():
         return None
 
     # Starting with Python 3.4, the names of parameters are mangled in
@@ -547,7 +557,7 @@ def _wrapFunctionWithSpecialNestedArgs(
         iter_vars.append(iter_var)
 
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=iter_var,
                 source=ExpressionBuiltinIter1(value=source, source_ref=source_ref),
                 source_ref=source_ref,
@@ -560,7 +570,7 @@ def _wrapFunctionWithSpecialNestedArgs(
                 outer_body.getLocalsScope().registerProvidedVariable(arg_var)
 
                 statements.append(
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=arg_var,
                         source=ExpressionSpecialUnpack(
                             value=ExpressionTempVariableRef(
