@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,6 @@ Namespace packages of Python3.3 or higher
 import os
 
 from nuitka import Options
-from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
 from nuitka.nodes.AttributeNodes import makeExpressionAttributeLookup
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
@@ -35,17 +34,20 @@ from nuitka.nodes.ContainerMakingNodes import (
 from nuitka.nodes.DictionaryNodes import StatementDictOperationSet
 from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
-    ExpressionImportModuleNameHard,
     ExpressionImportName,
     makeExpressionImportModuleFixed,
+    makeExpressionImportModuleNameHard,
 )
 from nuitka.nodes.ModuleAttributeNodes import (
     ExpressionModuleAttributeFileRef,
     ExpressionNuitkaLoaderCreation,
 )
 from nuitka.nodes.ModuleNodes import CompiledPythonPackage
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
-from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
+from nuitka.nodes.SubscriptNodes import makeExpressionIndexLookup
+from nuitka.nodes.VariableNameNodes import (
+    ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
+)
 from nuitka.PythonVersions import python_version
 
 from .TreeHelpers import makeStatementsSequenceFromStatement
@@ -55,8 +57,11 @@ from .VariableClosure import completeVariableClosures
 def _makeCall(module_name, import_name, attribute_name, source_ref, *args):
     return ExpressionCallNoKeywords(
         called=makeExpressionAttributeLookup(
-            expression=ExpressionImportModuleNameHard(
-                module_name=module_name, import_name=import_name, source_ref=source_ref
+            expression=makeExpressionImportModuleNameHard(
+                module_name=module_name,
+                import_name=import_name,
+                module_guaranteed=True,
+                source_ref=source_ref,
             ),
             attribute_name=attribute_name,
             source_ref=source_ref,
@@ -87,8 +92,11 @@ def getNameSpacePathExpression(package, source_ref):
         elements = [
             ExpressionCallNoKeywords(
                 called=makeExpressionAttributeLookup(
-                    expression=ExpressionImportModuleNameHard(
-                        module_name="os", import_name="path", source_ref=source_ref
+                    expression=makeExpressionImportModuleNameHard(
+                        module_name="os",
+                        import_name="path",
+                        module_guaranteed=True,
+                        source_ref=source_ref,
                     ),
                     attribute_name="dirname",
                     source_ref=source_ref,
@@ -207,14 +215,17 @@ def createNamespacePackage(module_name, is_top, source_ref):
 
 def createImporterCacheAssignment(package, source_ref):
     return StatementDictOperationSet(
-        dict_arg=ExpressionImportModuleNameHard(
-            module_name="sys", import_name="path_importer_cache", source_ref=source_ref
+        dict_arg=makeExpressionImportModuleNameHard(
+            module_name="sys",
+            import_name="path_importer_cache",
+            module_guaranteed=True,
+            source_ref=source_ref,
         ),
-        key=ExpressionSubscriptLookup(
+        key=makeExpressionIndexLookup(
             expression=ExpressionVariableNameRef(
                 provider=package, variable_name="__path__", source_ref=source_ref
             ),
-            subscript=makeConstantRefNode(constant=0, source_ref=source_ref),
+            index_value=0,
             source_ref=source_ref,
         ),
         value=ExpressionNuitkaLoaderCreation(provider=package, source_ref=source_ref),
