@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,11 +24,6 @@ Only after this is executed, variable reference nodes can be considered
 complete.
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementDelVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.FunctionNodes import MaybeLocalVariableUsage
 from nuitka.nodes.LocalsDictNodes import (
     ExpressionLocalsVariableRef,
@@ -41,6 +36,11 @@ from nuitka.nodes.NodeMakingHelpers import (
     mergeStatements,
 )
 from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import (
+    StatementReleaseVariable,
+    makeStatementDelVariable,
+)
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     makeExpressionVariableRef,
@@ -167,7 +167,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
                     statements = mergeStatements(
                         statements=(
-                            StatementAssignmentVariable(
+                            makeStatementAssignmentVariable(
                                 variable=tmp_variable,
                                 source=node.subnode_source.subnode_left,
                                 source_ref=node.source_ref,
@@ -175,7 +175,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                             makeTryFinallyStatement(
                                 provider=provider,
                                 tried=(
-                                    StatementAssignmentVariable(
+                                    makeStatementAssignmentVariable(
                                         variable=tmp_variable,
                                         source=makeExpressionOperationBinaryInplace(
                                             left=ExpressionTempVariableRef(
@@ -222,7 +222,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     variable_name=variable_name
                 )
 
-                new_node = StatementAssignmentVariable(
+                new_node = makeStatementAssignmentVariable(
                     variable=variable,
                     source=node.subnode_source,
                     source_ref=node.source_ref,
@@ -253,7 +253,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     variable_name=variable_name
                 )
 
-                new_node = StatementDelVariable(
+                new_node = makeStatementDelVariable(
                     variable=variable,
                     tolerant=node.tolerant,
                     source_ref=node.source_ref,
